#include "StdAfx.h"
#include ".\contact.h"
//------------------------------------------------------------------------------

// ****** CAddress class ******

void CAddress::SetCity(const CString& sValue)
{
	m_sCity = sValue;
}

CString	CAddress::GetCity()
{
	return m_sCity;
}


// ISerializable interface
int	CAddress::GetProperties(CStringList& properties)
{
	int nProperties = 1;
	properties.AddTail(_T("City"));
	return nProperties;
}

bool CAddress::GetPropertyValue(const CString& sProperty, CProperty& sValue)
{
	if(sProperty == _T("City"))
	{
		sValue = m_sCity;
		return true;
	}
	return false;
}

bool CAddress::SetPropertyValue(const CString& sProperty, CProperty& sValue)
{
	if(sProperty == _T("City"))
	{
		m_sCity = sValue;
		return true;
	}
	return false;
}

bool CAddress::HasMultipleInstances()
{
	return false;
}

CString CAddress::GetClassName()
{
	return _T("Address");
}

CString CAddress::GetID()
{
	return _T("");
}

// IObjectFactory
ISerializable* CAddress::Create()
{
	return new CAddress();
}

void CAddress::Destroy(ISerializable* obj)
{
	delete obj;
	obj = NULL;
}

//------------------------------------------------------------------------------
// ***** CContact class *****

CContact::CContact(void)
{
	m_sId			= _T("001");
	m_sFirstName	= _T("Siddharth");
	m_address.SetCity(_T(""));

	CAddress* address1 = new CAddress();
	address1->SetCity(_T("Bangalore"));
	m_addresses.AddTail(address1);

	CAddress* address2 = new CAddress();
	address2->SetCity(_T("Calcutta"));
	m_addresses.AddTail(address2);
}
//------------------------------------------------------------------------------

CContact::~CContact(void)
{
	POSITION pos = m_addresses.GetHeadPosition();

	while(pos)
	{
		CAddress* pAddress = (CAddress*)m_addresses.GetNext(pos);
		if(pAddress)
		{
			delete pAddress;
		}
	}
}
//------------------------------------------------------------------------------

void CContact::SetFirstName(const CString& sValue)
{
	m_sFirstName = sValue;
}
//------------------------------------------------------------------------------

CString CContact::GetFirstName()
{
	return m_sFirstName;
}
//------------------------------------------------------------------------------

void CContact::SetId(const CString& sValue)
{
	m_sId = sValue;
}
//------------------------------------------------------------------------------

CString CContact::GetId()
{
	return m_sId;
}
//------------------------------------------------------------------------------

CAddress* CContact::GetAddress()
{
	return &m_address;
}

CStringList* CContact::GetEmailIds()
{
	return &m_emails;
}

CPtrList* CContact::GetAddressList()
{
	return &m_addresses;
}

// ISerializable interface
int	CContact::GetProperties(CStringList& properties)
{	
	properties.AddHead(_T("FirstName"));
	properties.AddHead(m_address.GetClassName());
	properties.AddHead(_T("EmailId"));
	properties.AddHead(_T("XAddress"));
	return (int)properties.GetCount();
}
//------------------------------------------------------------------------------

// Used during serlization
bool CContact::GetPropertyValue(const CString& sProperty, CProperty& property)
{
	if(sProperty == _T("FirstName"))
	{
		property = m_sFirstName;
		return true;
	}
	else if(sProperty == m_address.GetClassName())
	{
		property = (ISerializable*)&m_address;
		property.SetFactory(&m_address); // IMP
		return true;
	}
	else if(sProperty == _T("EmailId"))
	{
		property = m_emails;
		return true;
	}
	else if(sProperty == _T("XAddress"))
	{
		property = m_addresses;
		property.SetFactory(&m_address); // IMP
		return true;
	}

	return false; // this property does not exist
}
//------------------------------------------------------------------------------

// Used during de-serialization
bool CContact::SetPropertyValue(const CString& sProperty, 
								CProperty& property)
{
	if(sProperty == _T("FirstName"))
	{
		m_sFirstName = property;
		return true;
	}
	else if(sProperty == _T("ID"))
	{
		m_sId = property;
		return true;
	}
	else if(sProperty == m_address.GetClassName())
	{
		CAddress* address = (CAddress*)(property.GetObject());
		m_address.SetCity(address->GetCity());
		
		// delete the passed in object if we don't need it
		property.GetFactory()->Destroy(address);
		
		return true;
	}
	else if(sProperty == _T("EmailId"))
	{
		CProperty::CopyStringList(m_emails, property.GetStringList());
		return true;
	}
	else if(sProperty == _T("XAddress"))
	{
		// first free any existing objects
		POSITION pos = m_addresses.GetHeadPosition();

		while(pos)
		{
			CAddress* pAddress = (CAddress*)m_addresses.GetNext(pos);
			delete pAddress;
		}
		
		CProperty::CopyPtrList(m_addresses, property.GetObjectList());
		return true;
	}
	
	return false; // this property does not exist
}
//------------------------------------------------------------------------------

bool CContact::HasMultipleInstances()
{
	return true; // we will have more than one contact instance
}
//------------------------------------------------------------------------------

CString CContact::GetClassName()
{
	return _T("Contact");
}
//------------------------------------------------------------------------------

CString CContact::GetID()
{
	return m_sId;
}
//------------------------------------------------------------------------------

// IObjectFactory Interface

ISerializable* CContact::Create()
{
	return new CContact();
}
//------------------------------------------------------------------------------

void CContact::Destroy(ISerializable* obj)
{
	delete obj;
}
//------------------------------------------------------------------------------